@@ -4,14 +4,13 @@
 #include "parser/token.h"
 #include "parser/lexer.h"
 #include "parser/keywords.h"
-#include "log.h"
 #include <QHash>
 #include <QPair>
 #include <QString>
 #include <QDebug>
 #include <QMetaType>
 
-QString invalidIdCharacters = "[]()\"'@*.,+-=/%&|:; \t\n<>";
+QString invalidIdCharacters = "[](){}\"'@*.,+-=/#$%&|:; \t\n<>";
 QHash<NameWrapper,QPair<QChar,QChar>> wrapperChars;
 QHash<NameWrapper,QPair<QChar,bool>> wrapperEscapedEnding;
 QList<NameWrapper> sqlite3Wrappers;
@@ -41,10 +40,15 @@ bool doesObjectNeedWrapping(const QString& str)
     if (str.isEmpty())
         return true;
 
-    if (isObjWrapped(str))
-        return false;
+    // It used to return false if object name looked to be wrapped already,
+    // but actually name [abc] is proper name that needs to be wrapped (i.e. "[abc]").
+    // Bug reported for this was #4362
+    //if (isObjWrapped(str))
+    //    return false;
 
-    if (isKeyword(str))
+    // The "soft keyword" check added, as they don't require wrapping.
+    // For example: SELECT replace('abc', 'a', '1');
+    if (isKeyword(str) && !isSoftKeyword(str))
         return true;
 
     for (int i = 0; i < str.size(); i++)
@@ -580,12 +584,80 @@ QString getQueryWithPosition(const QStringList& queries, int position, int* star
     return QString();
 }
 
+int getCursorFinalPositionForQueries(const QString& queries, int position)
+{
+    const static QSet<QChar> whitespaceChars = {' ', '\t'};
+
+    int len = queries.length();
+    if (position >= len || position < 1)
+        return position;
+
+    if (!whitespaceChars.contains(queries[position]) && queries[position] != '\xa')
+        return position;
+
+    // First checking characters at & after cursor - are they just whitespaces until end of line?
+    int newPos = position;
+    while (newPos < len)
+    {
+        QChar c = queries[newPos++];
+        if (whitespaceChars.contains(c))
+            continue;
+        else if (c == '\xa')
+            break;
+        else
+            return position;
+    }
+
+    // Okay, only whitespaces after cursor, so let's check whats before cursor,
+    // until the ';' - is it just whitespaces too?
+    newPos = position;
+    while (newPos > 1 && whitespaceChars.contains(queries[newPos - 1]))
+        newPos--;
+
+    if (queries[newPos - 1] == ';')
+    {
+        // Jackpot! Our cursor is just somewhere in whitespaces after the query.
+        // Let's consider that prior query as current.
+        return newPos - 1;
+    }
+
+    return position;
+}
+
 QString getQueryWithPosition(const QString& queries, int position, int* startPos)
 {
+    position = getCursorFinalPositionForQueries(queries, position);
     QStringList queryList = splitQueries(queries);
     return getQueryWithPosition(queryList, position, startPos);
 }
 
+QPair<int, int> getQueryBoundriesForPosition(const QString& contents, int cursorPosition, bool fallBackToPreviousIfNecessary)
+{
+    int queryStartPos;
+    QString query = getQueryWithPosition(contents, cursorPosition, &queryStartPos);
+    TokenList tokens = Lexer::tokenize(query);
+    tokens.trim();
+    tokens.trimRight(Token::OPERATOR, ";");
+
+    if (tokens.size() == 0 && fallBackToPreviousIfNecessary)
+    {
+        // Fallback
+        cursorPosition = contents.lastIndexOf(";", cursorPosition - 1);
+        if (cursorPosition > -1)
+        {
+            query = getQueryWithPosition(contents, cursorPosition, &queryStartPos);
+            tokens = Lexer::tokenize(query);
+            tokens.trim();
+            tokens.trimRight(Token::OPERATOR, ";");
+        }
+    }
+
+    if (tokens.size() == 0)
+        return QPair<int, int>(-1, -1);
+
+    return QPair<int, int>(tokens.first()->start + queryStartPos, tokens.last()->end + 1 + queryStartPos);
+}
+
 QString trimBindParamPrefix(const QString& param)
 {
     if (param == "?")
@@ -822,3 +894,15 @@ SqliteDataType toSqliteDataType(const QString& typeStr)
 
     return SqliteDataType::UNKNOWN;
 }
+
+QByteArray blobFromLiteral(const QString& value)
+{
+    if (value.length() <= 3)
+    {
+        qCritical() << "Call to blobFromLiteral() with blob literal shorter or equal to 3 characters:" << value;
+        return QByteArray();
+    }
+
+    QString hex = value.mid(2, value.length() - 3);
+    return QByteArray::fromHex(hex.toLatin1());
+}
