@@ -1,5 +1,5 @@
 #include "queryexecutor.h"
-#include "sqlerrorresults.h"
+#include "db/queryexecutorsteps/queryexecutorfilter.h"
 #include "sqlerrorcodes.h"
 #include "services/dbmanager.h"
 #include "db/sqlerrorcodes.h"
@@ -10,7 +10,6 @@
 #include "queryexecutorsteps/queryexecutorattaches.h"
 #include "queryexecutorsteps/queryexecutorcountresults.h"
 #include "queryexecutorsteps/queryexecutorexecute.h"
-#include "queryexecutorsteps/queryexecutorcellsize.h"
 #include "queryexecutorsteps/queryexecutorlimit.h"
 #include "queryexecutorsteps/queryexecutororder.h"
 #include "queryexecutorsteps/queryexecutorwrapdistinctresults.h"
@@ -49,7 +48,7 @@ QueryExecutor::QueryExecutor(Db* db, const QString& query, QObject *parent) :
     setAutoDelete(false);
 
     connect(this, SIGNAL(executionFailed(int,QString)), this, SLOT(cleanupAfterExecFailed(int,QString)));
-    connect(DBLIST, SIGNAL(dbAboutToBeUnloaded(Db*, DbPlugin*)), this, SLOT(cleanupBeforeDbDestroy(Db*)));
+    connect(DBLIST, SIGNAL(dbAboutToBeUnloaded(Db*,DbPlugin*)), this, SLOT(cleanupBeforeDbDestroy(Db*)));
     connect(DBLIST, SIGNAL(dbRemoved(Db*)), this, SLOT(cleanupBeforeDbDestroy(Db*)));
     connect(simpleExecutor, &ChainExecutor::finished, this, &QueryExecutor::simpleExecutionFinished, Qt::DirectConnection);
 }
@@ -82,6 +81,12 @@ void QueryExecutor::setupExecutionChain()
     executionChain.append(additionalStatelessSteps[AFTER_REPLACED_VIEWS]);
     executionChain.append(createSteps(AFTER_REPLACED_VIEWS));
 
+    executionChain << new QueryExecutorFilter()
+                   << new QueryExecutorParseQuery("after Filter");
+
+    executionChain.append(additionalStatelessSteps[AFTER_COLUMN_FILTERS]);
+    executionChain.append(createSteps(AFTER_COLUMN_FILTERS));
+
     executionChain << new QueryExecutorAddRowIds()
                    << new QueryExecutorParseQuery("after AddRowIds");
 
@@ -105,14 +110,8 @@ void QueryExecutor::setupExecutionChain()
     executionChain.append(additionalStatelessSteps[AFTER_DISTINCT_WRAP]);
     executionChain.append(createSteps(AFTER_DISTINCT_WRAP));
 
-    executionChain << new QueryExecutorCellSize()
-                   << new QueryExecutorCountResults()
-                   << new QueryExecutorParseQuery("after CellSize");
-
-    executionChain.append(additionalStatelessSteps[AFTER_CELL_SIZE_LIMIT]);
-    executionChain.append(createSteps(AFTER_CELL_SIZE_LIMIT));
-
-    executionChain << new QueryExecutorColumnType()
+    executionChain << new QueryExecutorCountResults()
+                   << new QueryExecutorColumnType()
                    << new QueryExecutorParseQuery("after ColumnType");
 
     executionChain.append(additionalStatelessSteps[AFTER_COLUMN_TYPES]);
@@ -130,13 +129,13 @@ void QueryExecutor::setupExecutionChain()
 
     executionChain << new QueryExecutorExecute();
 
-    for (QueryExecutorStep* step : executionChain)
+    for (QueryExecutorStep*& step : executionChain)
         step->init(this, context);
 }
 
 void QueryExecutor::clearChain()
 {
-    for (QueryExecutorStep* step : executionChain)
+    for (QueryExecutorStep*& step : executionChain)
     {
         if (!allAdditionalStatelsssSteps.contains(step))
             delete step;
@@ -149,7 +148,7 @@ void QueryExecutor::executeChain()
 {
     // Go through all remaining steps
     bool result;
-    for (QueryExecutorStep* currentStep : executionChain)
+    for (QueryExecutorStep*& currentStep : executionChain)
     {
         if (isInterrupted())
         {
@@ -189,7 +188,9 @@ void QueryExecutor::stepFailed(QueryExecutorStep* currentStep)
 
     if (isInterrupted())
     {
+        executionMutex.lock();
         executionInProgress = false;
+        executionMutex.unlock();
         emit executionFailed(SqlErrorCode::INTERRUPTED, tr("Execution interrupted."));
         return;
     }
@@ -416,7 +417,7 @@ QList<QueryExecutor::ResultRowIdColumnPtr> QueryExecutor::getRowIdResultColumns(
 int QueryExecutor::getMetaColumnCount() const
 {
     int count = 0;
-    for (ResultRowIdColumnPtr rowIdCol : context->rowIdColumns)
+    for (ResultRowIdColumnPtr& rowIdCol : context->rowIdColumns)
         count += rowIdCol->queryExecutorAliasToColumn.size();
 
     return count;
@@ -450,7 +451,9 @@ void QueryExecutor::executeSimpleMethod()
     if (queriesForSimpleExecution.isEmpty())
         queriesForSimpleExecution = quickSplitQueries(originalQuery, false, true);
 
-    QStringList queriesWithPagination = applyLimitForSimpleMethod(queriesForSimpleExecution);
+    QStringList queriesWithPagination = applyFiltersAndLimitAndOrderForSimpleMethod(queriesForSimpleExecution);
+    if (isExecutorLoggingEnabled())
+        qDebug() << "Simple Execution Method query:" << queriesWithPagination.join("; ");
 
     simpleExecutor->setQueries(queriesWithPagination);
     simpleExecutor->setDb(db);
@@ -479,7 +482,7 @@ void QueryExecutor::simpleExecutionFinished(SqlQueryPtr results)
 
     ResultColumnPtr resCol;
     context->resultColumns.clear();
-    for (const QString& colName : results->getColumnNames())
+    for (QString& colName : results->getColumnNames())
     {
         resCol = ResultColumnPtr::create();
         resCol->displayName = colName;
@@ -558,7 +561,7 @@ bool QueryExecutor::simpleExecIsSelect()
 void QueryExecutor::cleanup()
 {
     Db* attDb = nullptr;
-    for (const QString& attDbName : context->dbNameToAttach.leftValues())
+    for (QString& attDbName : context->dbNameToAttach.leftValues())
     {
         attDb = DBLIST->getByName(attDbName, Qt::CaseInsensitive);
         if (attDbName.isNull())
@@ -597,29 +600,65 @@ bool QueryExecutor::handleRowCountingResults(quint32 asyncId, SqlQueryPtr result
     return true;
 }
 
-QStringList QueryExecutor::applyLimitForSimpleMethod(const QStringList &queries)
+QStringList QueryExecutor::applyFiltersAndLimitAndOrderForSimpleMethod(const QStringList &queries)
 {
+    static_qstring(filtersTpl, "SELECT * FROM (%1) WHERE %2");
     static_qstring(tpl, "SELECT * FROM (%1) LIMIT %2 OFFSET %3");
-    if (page < 0)
-        return queries; // no paging requested
+    static_qstring(sortTpl, "SELECT * FROM (%1) ORDER BY %2");
+    static_qstring(sortColTpl, "%1 %2");
+
+    if (page < 0 && sortOrder.isEmpty())
+        return queries;
 
     QStringList result = queries;
     QString lastQuery = queries.last();
 
     bool isSelect = false;
     getQueryAccessMode(lastQuery, &isSelect);
-    if (isSelect)
+
+    // FILTERS
+    QString filters = getFilters();
+    if (isSelect && !filters.isEmpty())
     {
-        result.removeLast();
-        result << tpl.arg(trimQueryEnd(lastQuery), QString::number(resultsPerPage), QString::number(page * resultsPerPage));
+        lastQuery = filtersTpl.arg(
+            trimQueryEnd(lastQuery),
+            filters
+            );
     }
+
+    // ORDER BY
+    if (!sortOrder.isEmpty())
+    {
+        QStringList cols;
+        for (QueryExecutor::Sort& sort : sortOrder)
+        {
+            cols << sortColTpl.arg(
+                QString::number(sort.column + 1), // in ORDER BY column indexes are 1-based
+                (sort.order == QueryExecutor::Sort::DESC) ? "DESC" : "ASC"
+                );
+        }
+        lastQuery = sortTpl.arg(trimQueryEnd(lastQuery), cols.join(", "));
+    }
+
+    // LIMIT
+    if (page >= 0 && isSelect)
+    {
+        lastQuery = tpl.arg(
+            trimQueryEnd(lastQuery),
+            QString::number(resultsPerPage),
+            QString::number(page * resultsPerPage)
+            );
+    }
+
+    result.removeLast();
+    result << lastQuery;
     return result;
 }
 
 QList<QueryExecutorStep*> QueryExecutor::createSteps(QueryExecutor::StepPosition position)
 {
     QList<QueryExecutorStep*> steps;
-    for (StepFactory* factory : additionalStatefulStepFactories[position])
+    for (StepFactory*& factory : additionalStatefulStepFactories[position])
         steps << factory->produceQueryExecutorStep();
 
     return steps;
@@ -714,7 +753,7 @@ void QueryExecutor::handleErrorsFromSmartAndSimpleMethods(SqlQueryPtr results)
     {
         QString match;
         QString replaceName;
-        for (const QString& attachName : context->dbNameToAttach.rightValues())
+        for (QString& attachName : context->dbNameToAttach.rightValues())
         {
             match = attachName + ".";
             replaceName = wrapObjIfNeeded(context->dbNameToAttach.valueByRight(attachName)) + ".";
@@ -752,7 +791,7 @@ bool QueryExecutor::wasDataModifyingQuery() const
 QList<DataType> QueryExecutor::resolveColumnTypes(Db* db, QList<QueryExecutor::ResultColumnPtr>& columns, bool noDbLocking)
 {
     QSet<Table> tables;
-    for (ResultColumnPtr col : columns)
+    for (ResultColumnPtr& col : columns)
         tables << Table(col->database, col->table);
 
     SchemaResolver resolver(db);
@@ -774,7 +813,7 @@ QList<DataType> QueryExecutor::resolveColumnTypes(Db* db, QList<QueryExecutor::R
     QList<DataType> datatypeList;
     Table t;
     SqliteCreateTable::Column* parsedCol = nullptr;
-    for (ResultColumnPtr col : columns)
+    for (ResultColumnPtr& col : columns)
     {
         t = Table(col->database, col->table);
         if (!parsedTables.contains(t))
@@ -969,3 +1008,13 @@ int qHash(QueryExecutor::SourceTable sourceTable)
     return qHash(sourceTable.database + "." + sourceTable.table + "/" + sourceTable.alias);
 }
 
+
+QString QueryExecutor::getFilters() const
+{
+    return filters;
+}
+
+void QueryExecutor::setFilters(const QString& newFilters)
+{
+    filters = newFilters;
+}
