@@ -2,10 +2,10 @@
 #define QUERYEXECUTOR_H
 
 #include "db/db.h"
-#include "parser/token.h"
-#include "selectresolver.h"
 #include "coreSQLiteStudio_global.h"
 #include "common/bistrhash.h"
+#include "parser/ast/sqlitequery.h"
+#include "parser/ast/sqlitequerytype.h"
 #include "datatype.h"
 #include <QObject>
 #include <QHash>
@@ -15,7 +15,6 @@
 /** @file */
 
 class Parser;
-class SqliteQuery;
 class QueryExecutorStep;
 class DbPlugin;
 class ChainExecutor;
@@ -174,6 +173,10 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
                                * in COMPOUND_SELECT case. To learn about common table expression statement,
                                * see http://sqlite.org/lang_with.html
                                */
+            VIEW_NOT_EXPANDED,/**<
+                               * The data cell comes from a VIEW that was not expanded (because there were
+                               * multi-level views), therefore it was impossible to get ROWID for the cell.
+                               */
         };
 
         /**
@@ -652,6 +655,14 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
              * message from smart execution.
              */
             QString errorMessageFromSmartExecution;
+
+            /**
+             * @brief Flag indicating whether views were replaced/expanded.
+             *
+             * In other words, this flag tells whether the ReplaceViews step of query executor
+             * was executed, or skipped (due to many levels of views). False = skipped.
+             */
+            bool viewsExpanded = false;
         };
 
         /**
@@ -679,8 +690,8 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
             AFTER_REPLACED_COLUMNS,     /**< After all columns have been explicitly listed in result list, together with unique alias names */
             AFTER_ORDER,                /**< After order clause was applied/modified */
             AFTER_DISTINCT_WRAP,        /**< After wrapping SELECT was added in case of DISTINCT or GROUP BY clauses were used */
-            AFTER_CELL_SIZE_LIMIT,      /**< After cell result size was limited to save memory usage */
             AFTER_COLUMN_TYPES,         /**< After typeof() result meta columns were added */
+            AFTER_COLUMN_FILTERS,       /**< After WHERE filters applied */
             AFTER_ROW_LIMIT_AND_OFFSET, /**< After LIMIT and ORDER clauses were added/modified. This is the last possible moment, directly ahead of final query execution */
             JUST_BEFORE_EXECUTION,      /**< Same as AFTER_ROW_LIMIT_AND_OFFSET */
             LAST                        /**< Same as AFTER_ROW_LIMIT_AND_OFFSET */
@@ -1077,6 +1088,9 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
         bool getNoMetaColumns() const;
         void setNoMetaColumns(bool value);
 
+        void setFilters(const QString& newFilters);
+        QString getFilters() const;
+
         void handleErrorsFromSmartAndSimpleMethods(SqlQueryPtr results);
 
         /**
@@ -1222,7 +1236,7 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
          */
         bool handleRowCountingResults(quint32 asyncId, SqlQueryPtr results);
 
-        QStringList applyLimitForSimpleMethod(const QStringList &queries);
+        QStringList applyFiltersAndLimitAndOrderForSimpleMethod(const QStringList &queries);
 
         /**
          * @brief Creates instances of steps for all registered factories for given position.
@@ -1347,6 +1361,13 @@ class API_EXPORT QueryExecutor : public QObject, public QRunnable
          */
         int dataLengthLimit = -1;
 
+        /**
+         * @brief Optional filters to apply to the query.
+         * If not empty, it will be appended to the WHERE clause at the very end of execution chain,
+         * skipping complex result colum analysis, etc.
+         */
+        QString filters;
+
         /**
          * @brief Limit of queries, after which simple mode is used.
          *
