@@ -9,12 +9,12 @@
 #include "parser/ast/sqlitecreatetable.h"
 #include "uiconfig.h"
 #include "datagrid/sqlqueryview.h"
-#include "datagrid/sqlqueryrownummodel.h"
 #include "services/dbmanager.h"
 #include "querygenerator.h"
 #include "parser/lexer.h"
 #include "common/compatibility.h"
 #include "mainwindow.h"
+#include "iconmanager.h"
 #include <QHeaderView>
 #include <QDebug>
 #include <QApplication>
@@ -89,7 +89,6 @@ void SqlQueryModel::executeQuery()
         return;
     }
 
-    sortOrder.clear();
     queryExecutor->setSkipRowCounting(false);
     queryExecutor->setSortOrder(sortOrder);
     queryExecutor->setPage(0);
@@ -222,7 +221,7 @@ QModelIndexList SqlQueryModel::findIndexes(int role, const QVariant& value, int
     return findIndexes(startIdx, endIdx, role, value, hits);
 }
 
-QModelIndexList SqlQueryModel::findIndexes(const QModelIndex& start, const QModelIndex& end, int role, const QVariant& value, int hits) const
+QModelIndexList SqlQueryModel::findIndexes(const QModelIndex& start, const QModelIndex& end, int role, const QVariant& value, int hits, bool stringApproximation) const
 {
     QModelIndexList results;
     bool allHits = hits < 0;
@@ -231,6 +230,7 @@ QModelIndexList SqlQueryModel::findIndexes(const QModelIndex& start, const QMode
     int toRow = end.row();
     int fromCol = start.column();
     int toCol = end.column();
+    QString stringVal = value.toString();
 
     for (int row = fromRow; row <= toRow && (allHits || results.count() < hits); row++)
     {
@@ -241,7 +241,7 @@ QModelIndexList SqlQueryModel::findIndexes(const QModelIndex& start, const QMode
                  continue;
 
             QVariant cellVal = data(idx, role);
-            if (value != cellVal)
+            if (value != cellVal && !(stringApproximation && stringVal == cellVal.toString()))
                 continue;
 
             results.append(idx);
@@ -312,9 +312,9 @@ QHash<AliasedTable, QList<SqlQueryItem*> > SqlQueryModel::groupItemsByTable(cons
     {
         if (item->getColumn())
         {
-            table.setDatabase(item->getColumn()->database.toLower());
-            table.setTable(item->getColumn()->table.toLower());
-            table.setTableAlias(item->getColumn()->tableAlias.toLower());
+            table.setDatabase(item->getColumn()->database);
+            table.setTable(item->getColumn()->table);
+            table.setTableAlias(item->getColumn()->tableAlias);
             itemsByTable[table] << item;
         }
         else
@@ -410,7 +410,7 @@ void SqlQueryModel::refreshGeneratedColumns(const QList<SqlQueryItem*>& items)
     for (auto resultIt = resultValues.begin(); resultIt != resultValues.end(); resultIt++)
     {
         SqlQueryItem* item = resultIt.key();
-        item->setValue(resultIt.value(), false, true);
+        item->setValue(resultIt.value(), true);
         item->setTextAlignment(findValueAlignment(resultIt.value(), item->getColumn()));
     }
 }
@@ -1087,13 +1087,8 @@ void SqlQueryModel::updateItem(SqlQueryItem* item, const QVariant& value, int co
 void SqlQueryModel::updateItem(SqlQueryItem* item, const QVariant& value, int columnIndex, const RowId& rowId, Qt::Alignment alignment)
 {
     SqlQueryModelColumnPtr column = columns[columnIndex];
-
-    // This should be equal at most, unless we have UTF-8 string, than there might be more bytes.
-    // If less, than it's not limited.
-    bool limited = value.toByteArray().size() >= cellDataLengthLimit;
-
     item->setJustInsertedWithOutRowId(false);
-    item->setValue(value, limited, true);
+    item->setValue(value, true);
     item->setColumn(column.data());
     item->setTextAlignment(alignment);
     item->setRowId(rowId);
@@ -1132,6 +1127,14 @@ RowId SqlQueryModel::getNewRowId(const RowId& currentRowId, const QList<SqlQuery
         for (SqlQueryItem* item : items)
         {
             col = item->getColumn();
+            QStringList tableRowIdColumns = tableToRowIdColumn[col->getAliasedTable()].values();
+            if (tableRowIdColumns.contains(col->column, Qt::CaseInsensitive))
+            {
+                RowId newRowId;
+                newRowId[col->column] = item->getValue();
+                return newRowId;
+            }
+
             if (isRowIdKeyword(col->column) || col->isRowIdPk())
             {
                 RowId newRowId;
@@ -1170,7 +1173,7 @@ QHash<QString, QVariantList> SqlQueryModel::toValuesGroupedByColumns(const QList
 {
     QHash<QString, QVariantList> values;
     for (SqlQueryItem* item : items)
-        values[item->getColumn()->displayName] << item->getFullValue();
+        values[item->getColumn()->displayName] << item->getValue();
 
     return values;
 }
@@ -1187,7 +1190,6 @@ bool SqlQueryModel::readColumns()
     tablesInUse.clear();
 
     // Reading column mapping for ROWID columns
-    int totalRowIdCols = 0;
     AliasedTable aliasedTable;
     DbAndTable dbAndTable;
     for (const QueryExecutor::ResultRowIdColumnPtr& resCol : queryExecutor->getRowIdResultColumns())
@@ -1205,7 +1207,6 @@ bool SqlQueryModel::readColumns()
         aliasedTable.setTable(resCol->table);
         aliasedTable.setTableAlias(resCol->tableAlias);
         tableToRowIdColumn[aliasedTable] = resCol->queryExecutorAliasToColumn;
-        totalRowIdCols += resCol->queryExecutorAliasToColumn.size();
     }
 
     // Reading column details (datatype, constraints)
@@ -1406,6 +1407,7 @@ void SqlQueryModel::handleExecFinished(SqlQueryPtr results)
         return;
     }
 
+    emit aboutToLoadResults();
     storeStep1NumbersFromExecution();
     if (!loadData(results))
         return;
@@ -1488,33 +1490,57 @@ void SqlQueryModel::itemValueEdited(SqlQueryItem* item)
     emit commitStatusChanged(getUncommittedItems().size() > 0);
 }
 
+void SqlQueryModel::repaintAllItems()
+{
+    QModelIndex startIdx = index(0, 0);
+    if (!startIdx.isValid())
+        return;
+
+    QModelIndex endIdx = index(rowCount() - 1, columnCount() - 1);
+    emit dataChanged(startIdx, endIdx, QVector<int>({Qt::DisplayRole, Qt::EditRole}));
+}
+
 void SqlQueryModel::changeSorting(int logicalIndex, Qt::SortOrder order)
 {
     if (!reloadAvailable)
         return;
 
+    QueryExecutor::SortList sortList = QueryExecutor::SortList();
+    if (logicalIndex > -1)
+        sortList = {QueryExecutor::Sort(order, logicalIndex)};
+
     queryExecutor->setSkipRowCounting(true);
-    queryExecutor->setSortOrder({QueryExecutor::Sort(order, logicalIndex)});
+    queryExecutor->setSortOrder(sortList);
     reloadInternal();
 }
 
 void SqlQueryModel::changeSorting(int logicalIndex)
 {
     Qt::SortOrder newOrder = Qt::AscendingOrder;
-    if (sortOrder.size() == 1)
+    if (sortOrder.size() != 1)
     {
-        switch (sortOrder.first().order)
-        {
-            case QueryExecutor::Sort::ASC:
-                newOrder = Qt::DescendingOrder;
-                break;
-            case QueryExecutor::Sort::DESC:
-                newOrder = Qt::AscendingOrder;
-                break;
-            case QueryExecutor::Sort::NONE:
-                newOrder = Qt::AscendingOrder;
-                break;
-        }
+        changeSorting(logicalIndex, newOrder);
+        return;
+    }
+
+    QueryExecutor::Sort singleOrder = sortOrder.first();
+    if (singleOrder.column != logicalIndex)
+    {
+        changeSorting(logicalIndex, newOrder);
+        return;
+    }
+
+    switch (singleOrder.order)
+    {
+        case QueryExecutor::Sort::ASC:
+            newOrder = Qt::DescendingOrder;
+            break;
+        case QueryExecutor::Sort::DESC:
+            logicalIndex = -1;
+            break;
+        case QueryExecutor::Sort::NONE:
+            newOrder = Qt::AscendingOrder;
+            break;
     }
     changeSorting(logicalIndex, newOrder);
 }
@@ -2014,6 +2040,12 @@ void SqlQueryModel::applyStringFilter(const QString& value)
     // For custom query this is not supported.
 }
 
+void SqlQueryModel::applyStrictFilter(const QString& value)
+{
+    UNUSED(value);
+    // For custom query this is not supported.
+}
+
 void SqlQueryModel::applyRegExpFilter(const QString& value)
 {
     UNUSED(value);
@@ -2026,6 +2058,12 @@ void SqlQueryModel::applyStringFilter(const QStringList& values)
     // For custom query this is not supported.
 }
 
+void SqlQueryModel::applyStrictFilter(const QStringList& values)
+{
+    UNUSED(values);
+    // For custom query this is not supported.
+}
+
 void SqlQueryModel::applyRegExpFilter(const QStringList& values)
 {
     UNUSED(values);
@@ -2087,56 +2125,6 @@ bool SqlQueryModel::isExecutionInProgress() const
     return queryExecutor->isExecutionInProgress();
 }
 
-void SqlQueryModel::loadFullDataForEntireRow(int row)
-{
-    int colCnt = columns.size();
-    SqlQueryItem *item = nullptr;
-    for (int col = 0; col < colCnt; col++)
-    {
-        item = itemFromIndex(row, col);
-        if (!item)
-            continue;
-
-        if (!item->isLimitedValue())
-            continue;
-
-        item->loadFullData();
-    }
-}
-
-void SqlQueryModel::loadFullDataForEntireColumn(int column)
-{
-    int rowCnt = rowCount();
-    SqlQueryItem *item = nullptr;
-    for (int row = 0; row < rowCnt; row++)
-    {
-        item = itemFromIndex(row, column);
-        if (!item)
-            continue;
-
-        if (!item->isLimitedValue())
-            continue;
-
-        item->loadFullData();
-    }
-}
-
-bool SqlQueryModel::doesColumnHaveLimitedValues(int column) const
-{
-    int rowCnt = rowCount();
-    SqlQueryItem *item = nullptr;
-    for (int row = 0; row < rowCnt; row++)
-    {
-        item = itemFromIndex(row, column);
-        if (!item)
-            continue;
-
-        if (item->isLimitedValue())
-            return true;
-    }
-    return false;
-}
-
 void SqlQueryModel::CommitUpdateQueryBuilder::clear()
 {
     database.clear();
@@ -2180,7 +2168,7 @@ QString SqlQueryModel::CommitUpdateQueryBuilder::build()
     int argIndex = 0;
     QString arg;
     QStringList assignments;
-    for (const QString& col : columns)
+    for (QString& col : columns)
     {
         arg = ":value_" + QString::number(argIndex++);
         assignmentArgs << arg;
