@@ -54,7 +54,7 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
         SqlQueryItem* itemFromIndex(const QModelIndex& index) const;
         SqlQueryItem* itemFromIndex(int row, int column) const;
         QModelIndexList findIndexes(int role, const QVariant &value, int hits = -1) const;
-        QModelIndexList findIndexes(const QModelIndex &start, const QModelIndex& end, int role, const QVariant &value, int hits = -1) const;
+        QModelIndexList findIndexes(const QModelIndex &start, const QModelIndex& end, int role, const QVariant &value, int hits = -1, bool stringApproximation = false) const;
         QList<SqlQueryItem*> findItems(int role, const QVariant &value, int hits = -1) const;
         QList<SqlQueryItem*> findItems(const QModelIndex &start, const QModelIndex& end, int role, const QVariant &value, int hits = -1) const;
         SqlQueryItem* findAnyInColumn(int column, int role, const QVariant &value) const;
@@ -63,9 +63,6 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
         int columnCount(const QModelIndex& parent = QModelIndex()) const;
         QVariant headerData(int section, Qt::Orientation orientation, int role) const;
         bool isExecutionInProgress() const;
-        void loadFullDataForEntireRow(int row);
-        void loadFullDataForEntireColumn(int column);
-        bool doesColumnHaveLimitedValues(int column) const;
         StrHash<QString> attachDependencyTables();
         void detachDependencyTables();
 
@@ -94,6 +91,15 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
          */
         virtual void applySqlFilter(const QString& value);
 
+        /**
+         * @brief Request for applying an "equals" filtering on a dataset.
+         * @param value Filter expression.
+         * Default implementation does nothing. Working implementation (i.e. for a table)
+         * should set the query to temporary value which respects given filter and reload the data.
+         * Filter passed to this method is meant to be treated as strict value to be compared against.
+         */
+        virtual void applyStrictFilter(const QString& value);
+
         /**
          * @brief Request for applying "LIKE" filtering on a dataset.
          * @param value Filter expression.
@@ -120,6 +126,14 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
          */
         virtual void applyStringFilter(const QStringList& values);
 
+        /**
+         * @brief Request for applying an "equals" filtering on a dataset.
+         * @param values Filter expressions per column.
+         * This is the same as applyStrictFilter(const QString&), but is used for per-column filtering,
+         * when user enters filtering expressions for each column sparately.
+         */
+        virtual void applyStrictFilter(const QStringList& values);
+
         /**
          * @brief Request for applying Regular Expression filtering on a dataset.
          * @param values Filter expressions per column.
@@ -501,6 +515,7 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
 
     public slots:
         void itemValueEdited(SqlQueryItem* item);
+        void repaintAllItems();
         void changeSorting(int logicalIndex, Qt::SortOrder order);
         void changeSorting(int logicalIndex);
         void firstPage();
@@ -536,6 +551,12 @@ class GUI_API_EXPORT SqlQueryModel : public QStandardItemModel
          */
         void executionSuccessful();
 
+        /**
+         * @brief Execution is finished and data is about to be loaded to model.
+         * Emitted every query execution, every data reloading and every page change.
+         */
+        void aboutToLoadResults();
+
         /**
          * @brief executionFailed
          * @param errorText
