@@ -56,6 +56,8 @@ SqlQueryModelColumn::EditionForbiddenReason SqlQueryModelColumn::convert(QueryEx
             return EditionForbiddenReason::DISTINCT_RESULTS;
         case QueryExecutor::ColumnEditionForbiddenReason::COMM_TAB_EXPR:
             return EditionForbiddenReason::COMMON_TABLE_EXPRESSION;
+        case QueryExecutor::ColumnEditionForbiddenReason::VIEW_NOT_EXPANDED:
+            return EditionForbiddenReason::VIEW_NOT_EXPANDED;
     }
     return static_cast<EditionForbiddenReason>(-1);
 }
@@ -83,6 +85,8 @@ QString SqlQueryModelColumn::resolveMessage(SqlQueryModelColumn::EditionForbidde
             return QObject::tr("Cannot edit columns that are result of common table expression statement (%1).").arg("WITH ... SELECT ...");
         case EditionForbiddenReason::GENERATED_COLUMN:
             return QObject::tr("Cannot edit table generated columns.");
+        case EditionForbiddenReason::VIEW_NOT_EXPANDED:
+            return QObject::tr("Cannot edit columns that are result of a view if the executed query reads from any multilevel views (i.e. a view that queries another view).");
     }
     qCritical() << "Reached null text message for SqlQueryModel::EditionForbiddenReason. This should not happen!";
     return QString();
@@ -176,6 +180,11 @@ bool SqlQueryModelColumn::isGenerated() const
     return getConstraints<ConstraintGenerated*>().size() > 0;
 }
 
+QString SqlQueryModelColumn::getAliasedName() const
+{
+    return this->alias.isEmpty() ? this->column : this->alias;
+}
+
 QList<SqlQueryModelColumn::ConstraintFk*> SqlQueryModelColumn::getFkConstraints() const
 {
     return getConstraints<ConstraintFk*>();
@@ -190,6 +199,11 @@ SqlQueryModelColumn::ConstraintDefault* SqlQueryModelColumn::getDefaultConstrain
     return list[0];
 }
 
+AliasedTable SqlQueryModelColumn::getAliasedTable() const
+{
+    return AliasedTable(database, table, tableAlias);
+}
+
 int qHash(SqlQueryModelColumn::EditionForbiddenReason reason)
 {
     return static_cast<int>(reason);
@@ -211,6 +225,11 @@ QDataStream&operator >>(QDataStream& in, SqlQueryModelColumn*& col)
 
 
 SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(const QString& column, SqliteCreateTable::ConstraintPtr tableConstraint)
+{
+    return create(column, tableConstraint.data());
+}
+
+SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(const QString& column, SqliteCreateTable::Constraint* tableConstraint)
 {
     Constraint* constr = nullptr;
     switch (tableConstraint->type)
@@ -222,10 +241,18 @@ SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(const Q
 
             constr = new ConstraintPk();
             constr->type = Type::PRIMARY_KEY;
+            dynamic_cast<ConstraintPk*>(constr)->multiColumns =
+                map<SqliteIndexedColumn*, QString>(tableConstraint->indexedColumns, [](SqliteIndexedColumn* idxCol) -> QString
+                {
+                    return idxCol->detokenize().trimmed();
+                });
             break;
         }
         case SqliteCreateTable::Constraint::UNIQUE:
         {
+            if (!tableConstraint->doesAffectColumn(column))
+                return nullptr;
+
             constr = new ConstraintUnique();
             constr->type = Type::UNIQUE;
             break;
@@ -268,6 +295,11 @@ SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(const Q
 }
 
 SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(SqliteCreateTable::Column::ConstraintPtr columnConstraint)
+{
+    return create(columnConstraint.data());
+}
+
+SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(SqliteCreateTable::Column::Constraint* columnConstraint)
 {
     Constraint* constr = nullptr;
     switch (columnConstraint->type)
@@ -328,6 +360,7 @@ SqlQueryModelColumn::Constraint* SqlQueryModelColumn::Constraint::create(SqliteC
         {
             ConstraintGenerated* generate = new ConstraintGenerated();
             generate->generatedType = columnConstraint->generatedType;
+            generate->expr = columnConstraint->expr ? columnConstraint->expr->detokenize() : QString();
             constr = generate;
             constr->type = Type::GENERATED;
             break;
@@ -377,6 +410,9 @@ QString SqlQueryModelColumn::ConstraintPk::getTypeString() const
 QString SqlQueryModelColumn::ConstraintPk::getDetails() const
 {
     QStringList detailList;
+    if (!multiColumns.isEmpty())
+        detailList << "("+multiColumns.join(", ")+")";
+
     if (autoIncrement)
         detailList << "AUTOINCREMENT";
 
@@ -384,7 +420,12 @@ QString SqlQueryModelColumn::ConstraintPk::getDetails() const
         detailList << QObject::tr("on conflict: %1", "data view tooltip").arg(sqliteConflictAlgo(onConflict));
 
     if (detailList.size() > 0)
-        return "("+detailList.join(", ")+")";
+    {
+        if (detailList.size() > 1)
+            return "("+detailList.join(", ")+")";
+        else
+            return detailList.join(", ");
+    }
 
     return "";
 }
@@ -503,8 +544,7 @@ QString SqlQueryModelColumn::ConstraintGenerated::getTypeString() const
 
 QString SqlQueryModelColumn::ConstraintGenerated::getDetails() const
 {
-    return "("+QObject::tr("generated column type: %1", "data view tooltip")
-            .arg(SqliteCreateTable::Column::Constraint::toString(generatedType))+")";
+    return QString("(%1) %2").arg(expr, SqliteCreateTable::Column::Constraint::toString(generatedType)).trimmed();
 }
 
 Icon* SqlQueryModelColumn::ConstraintGenerated::getIcon() const
