@@ -187,6 +187,7 @@ void ExportDialog::initQueryPage()
 
         return dbOk && queryOk;
     });
+    ui->queryEdit->setAlwaysEnforceErrorsChecking(true);
 
     connect(ui->queryEdit, SIGNAL(errorsChecked(bool)), ui->queryPage, SIGNAL(completeChanged()));
     connect(ui->queryEdit, SIGNAL(textChanged()), ui->queryPage, SIGNAL(completeChanged()));
@@ -619,7 +620,7 @@ void ExportDialog::updatePluginOptions(ExportPlugin* plugin, int& optionsRow)
 
     configMapper = new ConfigMapper(cfgMain);
     configMapper->bindToConfig(pluginOptionsWidget);
-    connect(configMapper, SIGNAL(modified()), this, SLOT(updateValidation()));
+    connect(configMapper, SIGNAL(modified(QWidget*)), this, SLOT(updateValidation()));
     plugin->validateOptions();
 }
 
